package example;

import Cluster.KMeansCluster;

/**
 * @author Marios Michailidis 2013
 * @Goal The purpose of this example class is to show you
 * how to use the K-Means Cluster algorithm from
 * the KazAClasses package. In this example we will find the best 3 clusters
 * for a set of 2 variables.
 */
public class KMeans_Cluster_Example {
	/**
	 * @param args This is the main method that runs the example
	 */
	public static void main(String[] args) {
	
		/*
		 * Initialize a double Array for the 2 variables 
		 */
		double variables [][]={{4,12},{10,12},{5,10},{8,7},{5,3},{5,4},
				{2,11},{5,4},{3,8},{6,2},{7,4},{10,8},{8,9},
				{10,9}};
		/*
		 *  Initiate the K-Means Cluster class
		 */
		
		KMeansCluster KMeans= new KMeansCluster();
		
		/*
		 *  Initiate the KMeansCluster  main method
		 *  that computes all the important statistics. First 
		 *  we need to specify 3 things as this is an iterative
		 *  process :
		 *  1) The number of clusters we want
		 *  2)The maximum iterations we allow the algorithm to run 
		 *  in case it has not found the optimum solution.
		 *  3)The precision-how strict we are in defining the optimum solution
		 */
		
		int number_of_custers=3;
		int maximum_iterations_to_stop=20;
		double precision=0.001;
		
		/*
		 * Now we run the method
		 */
		KMeans.CreateKClusters(variables, number_of_custers, maximum_iterations_to_stop, precision);
		
		/*
		 * Now we can get the details of the 3 clusters (cendroids)
		 * in terms of means.
		 */
		
		double means [][]=KMeans.Getcluster_means();
		
		/*
		 * We'll print the results of the 3
		 * cendroids.
		 */
        for (int i=0; i< means.length; i++) {
        	System.out.println("The Cendroid " + i + " has mean of variable 1: " +means[i][0] + " and mean of variable 2: " +means[i][1]);
        }
		/*
		 * It will print:
		 * The Cendroid 0 has mean of variable 1: 9.2 and mean of variable 2: 9.0
           The Cendroid 1 has mean of variable 1: 3.5 and mean of variable 2: 10.25
           The Cendroid 2 has mean of variable 1: 5.6 and mean of variable 2: 3.4
		 */
        
        /*
         * We can get the sum of the minimum euclidean distances
         */
        double minimum_euclidean_distances=KMeans.GetMinimumSquaredDifference();
        
        /*
         * Print it
         */
        System.out.println(" The minimum Euclidian Distance is " + minimum_euclidean_distances);
    	/*
		 * It will print:
		 * The minimum Euclidian Distance is 21.437838875988845
		 */ 
        
        /*
         * We can see the classification results
         * of each observation, that is the closest
         * cluster for each observation.
         */
        String Classification []=KMeans.GetClassification();
        
        /*
         * Ultimately we can get the probability of each observation
         * to belong in each cluster. That is similar to K-Means-Fuzzy
         * algorithm as a logic.
         */
        double probabilities_of_clusters[][]=KMeans.GetKFuzzyClassifixationProbabilities();
        
        /*
         * Let's print the probabilities along with the 
         * classification prediction.
         */
        for(int i=0; i <probabilities_of_clusters.length; i++ ) {
        	System.out.println("Observation: " + i + " with cluster 0: " + probabilities_of_clusters[i][0] + " with cluster 1: " + probabilities_of_clusters[i][1]  + " with cluster 2: " + probabilities_of_clusters[i][2] + ".Therefore prediction is cluster " +Classification[i] );
        }
        /*
         * It will print:
         * Observation: 0 with cluster 0: 0.2946791570835371 with cluster 1: 0.6317773401337922 with cluster 2: 0.0735435027826709.Therefore prediction is cluster 1
           Observation: 1 with cluster 0: 0.6911934919014816 with cluster 1: 0.30880650809851845 with cluster 2: 0.0.Therefore prediction is cluster 0
           Observation: 2 with cluster 0: 0.3235060021008948 with cluster 1: 0.4928462049846353 with cluster 2: 0.18364779291446984.Therefore prediction is cluster 1
           Observation: 3 with cluster 0: 0.43694237323836627 with cluster 1: 0.24502955069063204 with cluster 2: 0.31802807607100175.Therefore prediction is cluster 0
           Observation: 4 with cluster 0: 0.17264841035994474 with cluster 1: 0.1667651314644183 with cluster 2: 0.660586458175637.Therefore prediction is cluster 2
           Observation: 5 with cluster 0: 0.206283326284027 with cluster 1: 0.21303474306563963 with cluster 2: 0.5806819306503335.Therefore prediction is cluster 2
           Observation: 6 with cluster 0: 0.19153452790749317 with cluster 1: 0.698960622767704 with cluster 2: 0.10950484932480276.Therefore prediction is cluster 1
           Observation: 7 with cluster 0: 0.206283326284027 with cluster 1: 0.21303474306563963 with cluster 2: 0.5806819306503335.Therefore prediction is cluster 2
           Observation: 8 with cluster 0: 0.2236739242896788 with cluster 1: 0.48673055650851477 with cluster 2: 0.2895955192018064.Therefore prediction is cluster 1
           Observation: 9 with cluster 0: 0.17519392286259963 with cluster 1: 0.09277390928069044 with cluster 2: 0.7320321678567099.Therefore prediction is cluster 2
           Observation: 10 with cluster 0: 0.2830181718018749 with cluster 1: 0.16835299242106405 with cluster 2: 0.5486288357770611.Therefore prediction is cluster 2
           Observation: 11 with cluster 0: 0.5796578947486529 with cluster 1: 0.19243198661279717 with cluster 2: 0.22791011863854985.Therefore prediction is cluster 0
           Observation: 12 with cluster 0: 0.4971434462057295 with cluster 1: 0.29321520434421716 with cluster 2: 0.20964134945005333.Therefore prediction is cluster 0
           Observation: 13 with cluster 0: 0.6135983338863514 with cluster 1: 0.21060747706814995 with cluster 2: 0.17579418904549848.Therefore prediction is cluster 0
         */
	}

}
