package example;

import Tests.Gini;
import Tests.Make_Woe;


public class Make_Woe_Example {

	/**
	 * @author Marios Michailidis 2013
	 * @Goal The purpose of this class is to demonstrate how to convert
	 * a string [] Array to numeric via the use of Weights of Evidence transformation
	 * (assuming there is another binary Array) from the Make_Woe class in the KazAClasses package.
	 * Assuming that we have an String [] array that shows the size of the house a person is living
	 * and whether that person has defaulted or not on  his/her loan, then we can compute the woe and 
	 * the information value, a measure of statistical relationship between a categorical variable
	 * and another binary one.
	 */
	
	public static void main(String[] args) {
		
		/*
		 * Initialize the String [] array that stores the size of the house
		 * each person is living.
		 */
		
		String [] House_Size ={"Small","Small","Medium","Small","Small","Big","Medium",
				"Small","Medium","Medium","Small","Big","Small","Small","Medium","Big",
				"Small","Small","Medium","Small","Big","Small","Medium","Medium","Big",
				"Medium","Small","Medium","Medium","Small","Medium","Medium","Big",
				"Medium","Medium","Small","Medium","Medium","Big","Big","Medium","Small",
				"Big","Big","Medium","Big","Medium","Big","Small","Big","Medium","Big",
				"Big","Big","Medium","Medium","Big","Small","Big","Big","Big","Medium",
				"Big","Big","Medium","Medium","Big","Big","Small","Big","Big","Medium",
				"Big","Big","Small","Medium","Medium","Small","Medium","Medium","Big",
				"Small","Small","Medium","Small","Small","Big","Small","Small","Medium",
				"Medium","Small","Small","Medium","Medium","Small","Medium","Small",
				"Medium","Small","Big","Medium"};
		/*
		 * Initialize the String [] array that has 0 when a person has defaulted on his/her
		 * loan and 1 when the person hasn't
		 */
		
		String Binary_Target [] ={"1","0","0","1","0","0","0","1","0","1","0","1","0","0","0","1","0","0","1","0","1","0","0","1",
				"0","1","0","0","1","0","1","0","1","0","0","0","0","1","1","0","1","0","1","1","0","1","0","1","0","1","0","1","0","1","1","1","0",
				"1","1","1","0","1","0","1","0","1","1","1","0","1","0","1","1","0","1","0","1","1","0","1","1","0","1","0","0","0","1","0","1","1",
				"1","0","0","1","1","0","1","1","0","1","1","0"};
		
		/*
		 *  Initiate the Make_Woe class
		 */
		Make_Woe woe= new Make_Woe();
		
		/*
		 *  Initiate the Make_Woe main method
		 *  that computes all the important statistics
		 */
		woe.give_me_woe(Binary_Target, House_Size);

		/*
		 * We can now get :
		 * 1)the distinct values of the House_Size variable
		 * 2) The computed WoE.
		 */
	   String Distinct_Values []=	woe.get_woe_names_st();
	   double woe_values []=woe.get_woe_classes();
	   
	   /*
	    * We can print these 2.
	    */
	   for (int i=0; i < Distinct_Values.length; i++) {
		   System.out.println(Distinct_Values[i] + " has woe of " + woe_values[i]);
	   }
		   /*it will print
		    * Medium has woe of 0.01207258123426924
             Big has woe of 0.8545971628688153
             Small has woe of -0.8276780735175514
             The values indicate that Big houses are more likely NOT to default
             unlike with the Small (very negative value).
		    */
		   
		   /*
		    * We can also compute the information value. Remember:
		    * IV < 0.03 the variable is not predictive
              IV < 0.1 the variable is a weak predictor
              IV < 0.25 the variable is a medium predictor
              IV < 0.5 the variable is a strong predictor
              IV >= 0.5 the variable is very strong predictor
		    */
		   double Information_Value=woe.get_information_value();
		   System.out.println("Information value is : " + Information_Value);
		   
		   /*
		    * It will print:
		    * Information value is : 0.4127980604820021
		    * According to the rule-of-thumb bands, the size variable is
		    * as string predictor of determining whether someone will default.
		    */
		  
		   /*
		    * We can also convert the initial String array House_Size
		    * to numeric by replacing each value with the respective woe
		    * (e.g. replace where medium with 0.01207258123426924, where
		    * Big with 0.8545971628688153 and small with -0.8276780735175514
		    */
		   double converted_array_with_woe []=woe.get_woe_variable();
		   
		/*
		 * In addition we could check the Gini of this variable
		 * We start always by initiating the class and then feeding the 
		 * main method.
		 */
	  
		   Gini gini= new Gini();
			
		// The gini might come out negative based on which class the the target it has chosen to regard as "good"
			gini.getgenieString(House_Size, Binary_Target);
			
			System.out.println(" Gini :" + Math.abs(gini.getGini()) + "%");
		 /*
		  * It will print:
		  *  Gini :33.92307692307692%
		  */
	}

}
